@@ -3,30 +3,31 @@ Defines an L{AEEvent} indicating that the focus has changed.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
-import Base, Constants
+import Base
+from AEConstants import LAYER_FOCUS
 
 class FocusChange(Base.AccessEngineEvent):
   '''
   Event that fires when the focused accessible changes.
   
   This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor.EventMonitor} using the L{Constants.registerEventType} function
-  when this module is first imported. The L{AEMonitor.EventMonitor} can use this
+  an L{AEMonitor} using the L{Base.registerEventType} function
+  when this module is first imported. The L{AEMonitor} can use this
   information to build its menus.
   
   @ivar gained: Type of focus change: True if gained, False if lost
   @type gained: boolean
   '''
-  Constants.registerEventType('FocusChange', True)
+  Base.registerEventType('FocusChange', True)
   
   def __init__(self, por, gained, **kwargs):
     '''
@@ -79,9 +80,9 @@ class FocusChange(Base.AccessEngineEvent):
         - type: Whether focus was gained (True) or lost (False)
     @rtype: dictionary
     '''
-    return {'por':self.getPOR(), 'gained':self.gained}
+    return {'por' : self.getPOR(), 'gained' : self.gained}
     
-  def execute(self, tier_manager, **kwargs):
+  def execute(self, tier_manager, view_manager, **kwargs):
     '''
     Contacts the L{TierManager} so it can manage the focus change event.
     
@@ -89,8 +90,12 @@ class FocusChange(Base.AccessEngineEvent):
     @type tier_manager: L{TierManager}
     @param kwargs: Packed references to other managers not of interest here
     @type kwargs: dictionary
-    @return: Always True to indicate the event executed properly
+    @return: True if there is an active view, False if not to delay execution 
+      of this event until there is
     @rtype: boolean
     '''
-    tier_manager.manageEvent(self)
-    return True
\ No newline at end of file
+    if view_manager.getAEView() is None and self.layer == LAYER_FOCUS:
+      return False
+    else:
+      tier_manager.manageEvent(self)
+      return True
