@@ -13,7 +13,7 @@ interface on L{pyLinAcc.Accessible} objects.
 @author: Peter Parente
 @author: Brett Clippingdale
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -26,7 +26,7 @@ from AEEvent import *
 from AccAdapt import Adapter
 from LSRInterfaces import *
 from pyLinAcc import Interfaces, Constants
-import pyLinAcc
+import pyLinAcc, AEConstants
 
 # opt: AE to raw event mapping, only registers for raw events when desired
 AE_MAP = {ViewChange : ['window:activate', 'window:deactivate', 
@@ -37,12 +37,11 @@ AE_MAP = {ViewChange : ['window:activate', 'window:deactivate',
           ChildrenChange : ['object:children-changed'],
           PropertyChange : ['object:property-change'],
           SelectorChange : ['object:text-selection-changed',
-                            #'object:selection-changed',
+                            'object:selection-changed',
                             'object:active-descendant-changed'],
           # opt: only register for state events that might be worth reporting 
           # to a script for the time being, lots of state change processing
           # can really slow down the screen reader
-          # @todo: PP: fill these out and move to an adapter method
           StateChange : ['object:state-changed:expanded', 
                          'object:state-changed:checked',
                          'object:state-changed:enabled',
@@ -57,10 +56,12 @@ AE_MAP = {ViewChange : ['window:activate', 'window:deactivate',
           ScreenChange : ['object:bounds-changed',
                           'object:text-bounds-changed',
                           'object:visible-data-changed'],
-          MouseChange : ['mouse:abs', 'mouse:button']
+          MouseChange : ['mouse:abs', 'mouse:button'],
+          PrivateChange : ['keyboard:press']
           }
 
 EVENT_HANDLERS = {
+  'keyboard:press' : '_handleKeyPressEvent',
   'window:activate': '_handleViewChange',
   'focus' : '_handleFocusEvent',
   'object:state-changed:focused' : '_handleFocusEvent',
@@ -147,7 +148,7 @@ class DefaultEventHandlerAdapter(Adapter):
     @type collector: callable
     @param vm: Reference to the view manager that has information about the 
       current view and needs to store a reference to a new view
-    @rtype vm: L{ViewManager}
+    @type vm: L{ViewManager}
     '''
     try:
       # make sure we can get the event type and role
@@ -156,38 +157,30 @@ class DefaultEventHandlerAdapter(Adapter):
     except Exception:
       # bad event
       return
-    
+
     # get if this is a window
     is_win = role == Constants.ROLE_WINDOW
     
-    if et == 'activate':
-      if vm.setView(event.source):
+    if et == 'activate' and (vm.setRawView(event.source) or 
+                             not vm.getRawActive()):
         # only a view change if the event source wasn't already the active view
-        vm.setActivated(True)
-        collector(ViewChange(vm.getView(), ViewChange.GAINED))
-    elif vm.getView() is not None and et == 'deactivate':
+        vm.setRawActive(True)
+        collector(ViewChange(vm.getRawView(), AEConstants.EVENT_VIEW_GAINED))
+    elif vm.getRawView() is not None and et == 'deactivate':
       por = POR(event.source)
-      if por == vm.getView():
+      if por == vm.getRawView():
         # only unset the flag if another activate hasn't been received
-        vm.setActivated(False)
-      collector(ViewChange(por, ViewChange.LOST))
-    elif vm.getActivated() and is_win:
-      # the active application created a floating window
-      if et == 'create':
-        vm.setView(event.source)
-      elif et == 'destroy':
-        # @todo: PP: need to refocus last focused control or fix gail bug
-        pass
-    elif not vm.getActivated() and is_win:
+        vm.setRawActive(False)
+      collector(ViewChange(por, AEConstants.EVENT_VIEW_LOST))
+    elif not vm.getRawActive() and is_win:
       # some other application created a floating window
-      if et == 'create':
-        if vm.setView(event.source):
-          collector(ViewChange(vm.getView(), ViewChange.GAINED))
-          # forget about the last focus so that a focus event can be fired when
-          # the previous application is activated
-          DefaultEventHandlerAdapter.last_focus = None
+      if et == 'create' and vm.setRawView(event.source):
+        collector(ViewChange(vm.getRawView(), AEConstants.EVENT_VIEW_GAINED))
+        # forget about the last focus so that a focus event can be fired when
+        # the previous application is activated
+        DefaultEventHandlerAdapter.last_focus = None
       elif et == 'destroy':
-        collector(ViewChange(POR(event.source), ViewChange.LOST))
+        collector(ViewChange(POR(event.source), AEConstants.EVENT_VIEW_LOST))
 
   @pyLinAcc.errorToLookupError
   def getAEEvents(self, event, collector):
@@ -203,17 +196,31 @@ class DefaultEventHandlerAdapter(Adapter):
       Accepts N parameters of type L{AEEvent}.
     @type collector: callable
     '''
-    # abort immediately if the accessible has state defunct or just went defunct
-    # and is now a dummy CORBA object, or if it has a redundant object role
-    try:
-      acc = self.subject
-      ss = acc.getState()
-      role = acc.getRole()
-      if (ss.contains(Constants.STATE_DEFUNCT) or 
-          role == Constants.ROLE_REDUNDANT_OBJECT):
+    acc = self.subject
+    if acc is None:
+      try:
+        # handle the case where there is no subject, e.g. a device event
+        name = EVENT_HANDLERS[event.type.name]
+        method = getattr(self, name)
+      except (KeyError, AttributeError):
+        return
+      # always consider key events focused
+      focused = True
+    else:
+      try:
+        ss = acc.getState()
+        role = acc.getRole()
+        if (ss.contains(Constants.STATE_DEFUNCT) or 
+            role == Constants.ROLE_REDUNDANT_OBJECT):
+          # abort immediately if the accessible has state defunct or if it has 
+          # a redundant object role
+          return
+        # make a first guess as to whether the object is focused or not
+        focused = (ss.contains(Constants.STATE_FOCUSED) or
+                   ss.contains(Constants.STATE_ACTIVE))
+      except AttributeError:
+        # abort immediately if the CORBA object is dead
         return
-    except AttributeError:
-      return
     
     try:
       # try to hash against an exact method
@@ -235,8 +242,7 @@ class DefaultEventHandlerAdapter(Adapter):
 
     # call the method that handles this event
     # keyword arguments will be passed as-is to AEEvent constructors
-    events = method(event, focused=ss.contains(Constants.STATE_FOCUSED) or
-                    ss.contains(Constants.STATE_ACTIVE))
+    events = method(event, focused=focused)
 
     if events: 
       if method == self._handleFocusEvent:
@@ -421,11 +427,11 @@ class DefaultEventHandlerAdapter(Adapter):
     '''
     major = event.type.major
     if major.startswith('text'):
-      kind = ScreenChange.TEXT_BOUNDS
+      kind = AEConstants.EVENT_TEXT_BOUNDS
     elif major.startswith('bounds'):
-      kind = ScreenChange.OBJECT_BOUNDS
+      kind = AEConstants.EVENT_OBJECT_BOUNDS
     else:
-      kind = ScreenChange.VISIBLE_DATA
+      kind = AEConstants.EVENT_VISIBLE_DATA
     por = POR(self.subject, None, 0, incomplete=True)
     return (ScreenChange(por, kind, **kwargs),)
 
@@ -443,10 +449,28 @@ class DefaultEventHandlerAdapter(Adapter):
     '''
     major = event.type.major
     if major == 'abs':
-      return (MouseChange(MouseChange.MOVE, pos=(event.detail1, 
-                                                 event.detail2)),)
+      return (MouseChange(AEConstants.EVENT_MOUSE_MOVE, pos=(event.detail1, 
+                                                             event.detail2)),)
     elif event.type.minor.endswith('p'):
-      kind = MouseChange.PRESS
+      kind = AEConstants.EVENT_MOUSE_PRESS
     else:
-      kind = MouseChange.RELEASE
-    return (MouseChange(kind, button=int(event.type.minor[0])),)
\ No newline at end of file
+      kind = AEConstants.EVENT_MOUSE_RELEASE
+    return (MouseChange(kind, button=int(event.type.minor[0])),)
+  
+  def _handleKeyPressEvent(self, event, **kwargs):
+    '''
+    Creates an L{AEEvent.PrivateChange} event in response to a key press on
+    the keyboard. This event is kept private because L{Perk}s and L{Task}s 
+    should not be relying on key codes and key syms to trigger actions, but
+    rather using abstracted L{AEEvent.InputGesture} events and 
+    L{Task.InputTask}s.
+    
+    @param event: Raw keyboard event
+    @type event: L{pyLinAcc.Event.Event}
+    @param kwargs: Parameters to be passed to any created L{AEEvent}
+    @type kwargs: dictionary
+    @return: L{AEEvent.PrivateChange} event
+    @rtype: tuple of L{AEEvent}
+    '''
+    return (PrivateChange(PrivateChange.KEY_PRESS, sym=event.any_data[0], 
+                          code=event.detail2, mod=event.any_data[1]),)
