@@ -11,7 +11,7 @@ containing embedded objects.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
@@ -23,7 +23,7 @@ import re
 import pyLinAcc
 from DefaultNav import *
 from DefaultInfo import *
-from DefaultEventHandler import *
+from TextAdapter import *
 from LSRInterfaces import *
 from pyLinAcc import Interfaces, Constants
 
@@ -145,7 +145,7 @@ class HypertextNavAdapter(DefaultNavAdapter):
     if char >= count:
       # if it does not exist, raise IndexError
       raise IndexError
-    elif text.getCharacterAtOffset(char-1) != EMBED_VAL:
+    elif char != 0 and text.getCharacterAtOffset(char-1) != EMBED_VAL:
       # if we're currently on text, not an item, find the next embed
       char = _getNextEmbedCharOffset(acc, char-1)
       # continue with the remaining if statements using the new char
@@ -353,6 +353,7 @@ class HypertextAccInfoAdapter(DefaultAccInfoAdapter):
     
     @todo: PP: don't get all text first, check start index of next hyperlink
       when available
+    @todo: PP: support soft line wrap as item text within a hypertext container
 
     @return: Accessible text of requested item
     @rtype: string
@@ -376,7 +377,7 @@ class HypertextAccInfoAdapter(DefaultAccInfoAdapter):
       # slice up to the embed character
       return text[io:i]
     
-class HypertextEventHandlerAdapter(DefaultEventHandlerAdapter):
+class HypertextEventHandlerAdapter(TextEventHandlerAdapter):
   '''  
   Overrides L{DefaultEventHandlerAdapter} to create proper L{POR}s for 
   hypertext objects having embed characters. Expects the subject to be a raw
