@@ -5,21 +5,21 @@ monitors.
 @author: Larry Weiss
 @author: Peter Parente
 @author: Brett Clippingdale
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import logging, sys
-import AEOutput, AEInput, AEEvent, AEMonitor, UIRegistrar
+import AEConstants, AEOutput, AEInput, AEEvent, AEMonitor, UIRegistrar
 from LSRInterfaces import implements
 from UIRegistrar import DEVICE, MONITOR
 from i18n import _
-import LSRConstants
 
 log = logging.getLogger('Device')
 
@@ -42,14 +42,16 @@ class DeviceManager(object):
   @type in_mons: L{AEMonitor.MonitorCollection}
   @ivar out_mons: Collection of monitors to notify about output
   @type out_mons: L{AEMonitor.MonitorCollection}
-  @ivar def_output: The "default" output device
-  @type def_output: L{AEOutput}
-  @ivar def_input: The "default" input device
-  @type def_input: L{AEInput}
   @ivar acc_eng: The AccessEngine reference that created this class
   @type acc_eng: L{AccessEngine}
   @ivar marks: Maps index markers on devices to 
   @type marks: dictionary
+  @ivar temp_dev: List of L{AEOutput} and L{AEInput} devices last removed in 
+    L{unloadDevices} and held until the next call of L{loadDevices}. The 
+    purpose of this list is to ensure strong references to all devices exist,
+    and thus weak references are not destroyed, until the the L{DeviceManager}
+    is repopulated with devices.
+  @type temp_dev: list
   '''
   def __init__(self, acc_eng):
     '''
@@ -62,12 +64,11 @@ class DeviceManager(object):
     self.in_devs = []
     self.in_mons = AEMonitor.MonitorCollection()
     self.out_mons = AEMonitor.MonitorCollection()
-    self.def_output = None
-    self.def_input = None
     self.acc_eng = acc_eng
     self.event_manager = None
     self.settings_manager = None
     self.marks = {}
+    self.temp_out = []
 
   def init(self, event_man, sett_man, **kwargs):
     '''
@@ -110,22 +111,28 @@ class DeviceManager(object):
                  (d.getName(), str(e)))
     
     # warn if we didn't initialize at least one output device
-    if self.def_output is None:
+    if not len(self.out_devs):
       log.warn(_('no output available'))
-    if self.def_input is None:
+    if not len(self.in_devs):
       log.warn(_('no input available'))
-      
+    self.temp_devs = []
+
   def unloadDevices(self):
     '''
-    Unloads all current devices.
+    Unloads all devices from L{out_devs} and L{in_devs}, but keeps strong 
+    references to all objects in L{temp_out} until L{loadDevices} is called 
+    again. Keeping strong references ensures weak references in L{Perk}s 
+    continue to exist until the L{DeviceManager} has new devices available 
+    for I/O.
+    
+    @todo: PP: unload input devices too once input device modifiers can be
+      properly saved and restored
     '''
-    for d in self.out_devs:
+    for d in self.out_devs[:]:
+      self.temp_devs.append(d)
       self.unregisterDevice(d)
-    # @todo: PP: support unloading of input devices
     #for d in self.in_devs:
       #self.unregisterDevice(d)
-    self.def_output = None
-    #self.def_input = None
     
   def close(self):
     '''
@@ -133,152 +140,159 @@ class DeviceManager(object):
     devices and L{AEMonitor}s. Saves all style information for currently 
     loaded devices.
     '''
-    for d in self.out_devs:
+    for d in self.out_devs[:]:
       self.unregisterDevice(d)
-    for d in self.in_devs:
+    for d in self.in_devs[:]:
       self.unregisterDevice(d)
-    self.def_output = None
-    self.def_input = None
     self.in_mons.clear()
     self.out_mons.clear()
+    
+  def send(self, dev, name, value, sem, layer):
+    '''
+    Sends arbitrary data to a device. The device must recognize the name in
+    order to decide what to do with the value data.
+    
+    This is a generic method which receives all content and commands to be 
+    rendered and executed on a device. Standard name identifiers should be 
+    used whenever possible. For instance, L{AEConstants.CMD_STOP} and
+    L{AEConstants.CMD_TALK}. However, device specific names and values
+    are certainly possible.
+
+    @param name: Descriptor of the data value sent
+    @type name: object
+    @param value: Content value
+    @type value: object
+    @param sem: The semantic stream on which to send output; defaults to None 
+      for the default semantic.
+    @type sem: integer
+    @param layer: Layer on which the event occurred
+    @type layer: integer
+    @param dev: Device that should receive the command
+    @type dev: L{AEOutput}
+    @return: Return value specific to the given command
+    @rtype: object
+    '''
+    # notify monitors
+    self.out_mons.show(OutputEvent(name), value=value, dev=dev, sem=sem, 
+                       layer=layer)
+    # quit if no device specified
+    if dev is None: return
+    if sem is None or layer is None:
+      return dev.send(name, value, None)
+    else:
+      style = dev.getStyle((sem, layer))
+      # respect muting
+      if style.Mute: return
+      # send the data
+      rv = dev.send(name, value, style)
+      # mark the style as clean
+      style.makeClean()
+      return rv
 
   def sendStop(self, dev, sem, layer):
     '''
-    Sends the stop command to the referenced output device. When the device has
-    not been registered or is None, the stop command is sent to the default
-    output device.
+    Sends the stop command to the referenced output device.
 
-    @param dev: The device to send the stop command or None to send to L{the 
-      default output device<setDefaultOutput>}.
+    @param dev: Device that should receive the command
     @type dev: L{AEOutput}
     @param sem: Semantic description of the information to stop None to 
       indicate stopping all output
     @type sem: integer
-    @param layer: Layer on which the event occurred, one of 
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     '''
-    # when dev is None assume default output
-    dev = dev or self.def_output
     # notify monitors
-    self.out_mons.show(event=OutputEvent.STOP, dev=dev, sem=sem, layer=layer)
+    self.out_mons.show(OutputEvent(AEConstants.CMD_STOP), dev=dev, sem=sem, 
+                       layer=layer)
     if dev is None: return
     if sem is None and layer is None:
       # send a global stop
-      dev.sendStop(None)
+      dev.send(AEConstants.CMD_STOP, None, None)
     elif sem is None:
       # find all styles associated with the given layer
       all = set([dev.getStyle((sem, layer)) for sem in
-                 LSRConstants.SEMANTIC_STYLES.keys()])
-      map(dev.sendStop, all)
+                 AEConstants.SEMANTIC_STYLES.keys()])
+      for style in all:
+        dev.send(AEConstants.CMD_STOP, None, style)
     else:
-      s = dev.getStyle((sem, layer))
-      dev.sendStop(s)
+      style = dev.getStyle((sem, layer))
+      dev.send(AEConstants.CMD_STOP, None, style)
 
   def sendTalk(self, dev, sem, layer):
     '''
-    Tells the specified output device to send buffered data. When the device
-    has not been registered or is None, the talk command is sent to the
-    default output device.
+    Tells the specified output device to send buffered data.
 
-    @param dev: The device to send the talk command or None to send to L{the 
-      default output device<setDefaultOutput>}
+    @param dev: Device that should receive the command
     @type dev: L{AEOutput}
     @param sem: Semantic information to start outputting or None to indicate 
       that all buffered information should be output
     @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     '''
-    # when dev is None assume default output
-    dev = dev or self.def_output
     # notify monitors
-    self.out_mons.show(event=OutputEvent.TALK, dev=dev, sem=sem, layer=layer)
+    self.out_mons.show(OutputEvent(AEConstants.CMD_TALK), dev=dev, sem=sem, 
+                       layer=layer)
     if dev is None: return
     if sem is None:
-      s = None
+      style = None
     else:
-      s = dev.getStyle((sem, layer))
-    dev.sendTalk(s)
+      style = dev.getStyle((sem, layer))
+    # send the command
+    dev.send(AEConstants.CMD_TALK, None, style)
 
-  def sendFilename(self, dev, filename, sem, layer, talk=True):
+  def sendFilename(self, dev, filename, sem, layer):
     '''
-    Sends the filename to the specified output device. When the device has not
-    been registered or is None, the string is sent to the default output
-    device.
+    Sends the filename to the specified output device.
 
-    @param dev: The device to send the string or None to send to L{the default 
-      output device<setDefaultOutput>}
+    @param dev: Device that should receive the command
     @type dev: L{AEOutput}
     @param filename: The filename to send to the referenced device
     @type filename: string
     @param sem: Semantic description of the text to send or None to indicate the
       information is void of any known semantic
     @type sem: integer 
-    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
-    @param talk: Defaults to True to indicate that L{sendTalk} should be called
-      after all characters are sent
-    @type talk: boolean
     '''
-    # when dev is None assume default output
-    dev = dev or self.def_output
     # notify monitors
-    self.out_mons.show(event=OutputEvent.FILENAME, text=filename, dev=dev, 
-                       sem=sem, layer=layer)
+    self.out_mons.show(OutputEvent(AEOutput.NAME_FILENAME), value=filename, 
+                       dev=dev, sem=sem, layer=layer)
     if dev is None: return
     # find the style
     style = dev.getStyle((sem, layer))
     # respect muting
     if style.Mute: return
     try:
-      dev.sendFilename(filename, style)
-      if talk:
-        self.sendTalk(dev, sem, layer)
+      dev.send(AEConstants.CMD_FILENAME, filename, style)
     except NotImplementedError:
       pass
-    # make the style as clean
+    # make the style clean
     style.makeClean()
     
-  def sendString(self, dev, text, sem, layer, talk=True, por=None):
+  def sendString(self, dev, text, sem, layer, por=None):
     '''
-    Sends the string to the specified output device. When the device has not
-    been registered or is None, the string is sent to the default output
-    device. The string is sent with a L{AEOutput.Base.AEOutput.sendString} call
-    followed by L{sendTalk} when the talk parameter is True.
+    Sends the string to the specified output device.
     
-    @todo: PP: collect word markers
+    @todo: PP: send indices and collect word markers
 
-    @param dev: The device to send the string or None to send to L{the default 
-      output device<setDefaultOutput>}
+    @param dev: Device that should receive the command
     @type dev: L{AEOutput}
     @param text: The text to send to the referenced device
     @type text: string
-    @param sem: Semantic description of the text to send or None to indicate the
-      information is void of any known semantic
+    @param sem: Semantic description of the text to send or None to indicate 
+      the information is void of any known semantic
     @type sem: integer 
-    @param layer: Layer on which the event occurred, one of 
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
-    @param talk: Defaults to True to indicate that L{sendTalk} should be called
-      after all characters are sent
-    @type talk: boolean
     @param por: Point of regard to the start of the string if one exists or 
       None if the string did not originate from a L{POR}
     @type por: L{POR}
     '''
-    # when dev is None assume default output
-    dev = dev or self.def_output
     # notify monitors
-    self.out_mons.show(event=OutputEvent.STRING, text=text, dev=dev,
+    self.out_mons.show(OutputEvent(AEConstants.CMD_STRING), value=text, dev=dev, 
                        sem=sem, layer=layer)  
-    if dev is None:
-      # make sure we log the talk, even if no device exists
-      if talk:
-        self.sendTalk(dev, sem, layer)
-      return
+    if dev is None: return
     # get the style
     style = dev.getStyle((sem, layer))
     # respect muting
@@ -287,49 +301,36 @@ class DeviceManager(object):
     parsed = dev.parseString(text, style, por)
     # send all words, pors, and styles
     for word, por, new_style in parsed:
-      # send the index and collect the marker
-      #try:
-        #marker = dev.sendIndex(style)
-      #except NotImplementedError:
-        #pass
-      # send the word string
-      dev.sendString(word, new_style)
-    if talk:
-      self.sendTalk(dev, sem, layer)
-    # mark the style as clean
+      # @todo: PP: send the index and collect the marker
+      dev.send(AEConstants.CMD_STRING, word, new_style)
+      # mark the style as clean
+      new_style.makeClean()
+    # mark the style clean
     style.makeClean()
 
-  def sendIndex(self, dev, sem, layer, data=None):
+  def sendIndex(self, dev, value, sem, layer):
     '''
-    Sends the referenced index marker to the referenced device. When the
-    device has not been registered or is None, the index marker is sent to the
-    default output device.
+    Sends the referenced index marker to the referenced device.
 
-    @param dev: The device to send the index marker or None to send to L{the 
-      default output device<setDefaultOutput>}
+    @param dev: Device that should receive the command
     @type dev: L{AEOutput}
     @param sem: Semantic description of the index text to send or None to 
       indicate the information is void of any known semantic
     @type sem: integer
-    @param layer: Layer on which the event occurred, one of 
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
-    @param data: Data to be forwarded to any observers when the index is 
-      reached
-    @type data: object
     '''
-    # when dev is None assume default output
-    dev = dev or self.def_output
     # notify registered monitors
-    self.out_mons.show(event=OutputEvent.INDEX, dev=dev, sem=sem,layer=layer)
+    self.out_mons.show(OutputEvent(AEConstants.CMD_INDEX), dev=dev, sem=sem,
+                       layer=layer)
     if dev is None: return
     # get the style
-    s = dev.getStyle((sem, layer))
+    style = dev.getStyle((sem, layer))
     # respect muting
-    if s.Mute: return
+    if style.Mute: return
     try:
       # generate a new marker on the device
-      mark = dev.sendIndex(s)
+      mark = dev.send(AEConstants.CMD_INDEX, None, style)
     except NotImplementedError:
       # do nothing when indexing not implemented
       return
@@ -338,25 +339,26 @@ class DeviceManager(object):
 
   def sendWelcome(self):
     '''Outputs the LSR welcome message on the default output device.'''
-    msg = _('welcome to %s version %s, revision %s') % \
-        (LSRConstants.NAME, LSRConstants.PROG_VERSION, LSRConstants.PROG_DATE)
     try:
-      st = self.def_output.getStyle((LSRConstants.SEM_ITEM, 
-                                     AEEvent.LAYER_FOCUS))
-    except AttributeError:
+      dev = self.out_devs[0]
+    except IndexError:
       # no output available
       return
-    self.def_output.sendString(msg, st)
-    self.def_output.sendTalk()
+    msg = _('welcome to %s version %s, revision %s') % \
+        (AEConstants.NAME, AEConstants.PROG_VERSION, AEConstants.PROG_DATE)
+    st = dev.getStyle((AEConstants.SEM_ITEM, AEConstants.LAYER_FOCUS))
+    dev.send(AEConstants.CMD_STRING, msg, st)
+    dev.send(AEConstants.CMD_TALK, None)
     
   def sendGoodbye(self):
     '''Outputs the LSR goodbye message on the default output device.'''
     try:
-      st = self.def_output.getStyle((LSRConstants.SEM_ITEM, 
-                                     AEEvent.LAYER_FOCUS))
-      self.def_output.sendStringSync(_('lsr exiting'), st)
-    except AttributeError:
-      pass
+      dev = self.out_devs[0]
+    except IndexError:
+      # no output available
+      return
+    st = dev.getStyle((AEConstants.SEM_ITEM, AEConstants.LAYER_FOCUS))
+    dev.send(AEConstants.CMD_STRING_SYNC, _('lsr exiting'), st)
 
   def _isDuplicateDevice(self, dev, output):
     '''
@@ -369,16 +371,15 @@ class DeviceManager(object):
     @type output: boolean
     '''
     if output:
-      dt = AEOutput.DEVICE_TYPES
       reg_devs = self.out_devs
     else:
-      dt = AEInput.DEVICE_TYPES
       reg_devs = self.in_devs
     # get interfaces device implements
-    dev_int = (i for i in dt if implements(dev, i))
-    # check if some registered device already implements all of these interfaces
+    dev_int = dev.getCapabilities()
+    # check if some registered device already implements all of these 
+    # interfaces
     for reg_dev in reg_devs:
-      reg_int = (i for i in dt if implements(reg_dev, i))
+      reg_int = reg_dev.getCapabilities()
       if set(reg_int).issuperset(dev_int):
         return True
     return False
@@ -386,13 +387,13 @@ class DeviceManager(object):
   def _registerInputDevice(self, dev):
     '''
     Registers the given device as an input device if the device implements the
-    L{AEInput} base class and provides some subinterface not already provided
-    by another registered output device. For instance, if a device implementing
-    L{AEInput.SystemInput} is already registered, another device implementing 
-    just L{AEInput.SystemInput} will not be registered by this method.
+    L{AEInput} base class and provides some capabilities not already provided
+    by another registered input device. For instance, if a device reports it
+    supports the "system keyboard" capability, no other system keyboard device
+    will be loaded.
     
     @param dev: Device to attempt to register
-    @type dev: L{AEInput.AEInput} class
+    @type dev: L{AEInput.AEInput}
     @return: True if the device was registered, False if it is not an 
       L{AEInput} device, and None if it provides no new interfaces
     @rtype: boolean or None
@@ -412,21 +413,66 @@ class DeviceManager(object):
     # append to the in_devs list after successfully initialized
     self.in_devs.append(dev)
     log.debug('added input device %s', str(dev))
-    # first input device registered is default
-    if self.def_input is None:
-      self.def_input = dev
     return True
   
+  def _unregisterInputDevice(self, dev):
+    '''
+    Unregisters the given input device. Removes this manager from
+    the list of gesture listeners stored in the device.
+    
+    @todo: PP: let an ouput device save its state too?
+    
+    @param dev: Device to unregister
+    @type dev: L{AEInput.AEInput}
+    '''
+    try:
+      # remove from list of input devices      
+      self.in_devs.remove(dev)
+      log.debug('removed input device %s', dev.getName())
+    except ValueError:
+      pass
+    try:
+      # do not observe gestures
+      dev.removeInputListener(self._onGesture)
+    except AttributeError:
+      pass
+  
+  def _unregisterOutputDevice(self, dev):
+    '''
+    Unregisters the given output device. Provides the device with a reference 
+    to the L{SettingsManager} so it can save state. Removes this manager from
+    the list of index listeners stored in the device.
+    
+    @param dev: Device to unregister
+    @type dev: L{AEOutput.AEOutput}
+    '''
+    try:
+      # let an output device save its styles
+      dev.saveStyles(self.settings_manager)
+    except AttributeError:
+      pass
+    try:
+      # remove from list of output devices
+      self.out_devs.remove(dev)
+      log.debug('removed output device %s', dev.getName())
+    except ValueError:
+      pass
+    try:
+      # do not observe indices
+      dev.removeIndexListener(self._onIndex)
+    except AttributeError:
+      pass
+  
   def _registerOutputDevice(self, dev):
     '''
     Registers the given device as an output device if the device implements the
     L{AEOutput} base class and provides some subinterface not already provided
-    by another registered output device. For instance, if a device implementing
-    L{AEOutput.Audio} is already registered, another device implementing just
-    L{AEOutput.Audio} will not be registered by this method.
+    by another registered output device. For instance, if a device reports
+    having "audio" capability, no other device providing just this capability
+    will be loaded.
     
     @param dev: Device to attempt to register
-    @type dev: L{AEOutput.AEOutput} class
+    @type dev: L{AEOutput.AEOutput}
     @return: True if the device was registered, False if it is not an 
       L{AEOutput} device, and None if it provides no new interfaces
     @rtype: boolean or None
@@ -445,43 +491,50 @@ class DeviceManager(object):
     # add DeviceManager as an index listener when implemented
     try:
       dev.addIndexListener(self._onIndex)
-    except NotImplementedError:
-      # do nothing when indexing not implemented
+    except (AttributeError, NotImplementedError):
+      # do nothing when indexing not implemented or supported
       pass
     # append to the out_devs list after successfully initialized
     self.out_devs.append(dev)
     log.debug('added output device %s', str(dev))
     # first output device registered is default
-    if self.def_output is None:
-      self.def_output = dev
 
     try:
       # try to load styles for the device
-      dev.postInitOutput(self.settings_manager)
+      dev.loadStyles(self.settings_manager)
     except KeyError:
       # or initialize new output styles
       self._initOutputStyles(dev)
+    # finally, always call postInit to let the device do more initialization
+    # after the styles are available
+    dev.postInit()
     return True
   
   def _initOutputStyles(self, dev):
     '''
-    Initializes styles for an L{AEOutput} device. Calls createDistinctStyles
-    on the device to get an initial batch of styles to use to distinguish
-    some types of information.
+    Initializes styles for an L{AEOutput} device. Calls 
+    L{AEOutput.AEOutput.createDistinctStyles} on the device to get an initial 
+    batch of styles to use to distinguish some types of information. If that
+    method is not implemented, the exception is ignored. Future requests to
+    use styles per semantic tag will resort to making flyweights for the
+    default style on the device.
     
     @param dev: Device reference
     @type dev: L{AEOutput}
     '''
     # create styles based on the abilities of this device
-    num_layers = len(AEEvent.LAYERS_ALL)
-    num_groups = len(LSRConstants.STYLE_GROUP_ALL)
-    styles = dev.createDistinctStyles(num_groups, num_layers)
+    num_layers = len(AEConstants.LAYERS_ALL)
+    num_groups = len(AEConstants.STYLE_GROUP_ALL)
+    try:
+      styles = dev.createDistinctStyles(num_groups, num_layers)
+    except NotImplementedError:
+      return
     # create semantics, layer to styles mapping for device
-    for layer, i in enumerate(AEEvent.LAYERS_ALL):
+    for layer, i in enumerate(AEConstants.LAYERS_ALL):
       # provide a default style for a semantic of None
       s = styles[i*num_groups]
       dev.setStyle((None, layer), s)
-      for sem, grp in LSRConstants.SEMANTIC_STYLES.items():
+      for sem, grp in AEConstants.SEMANTIC_STYLES.items():
         # provide a style for (semantic, layer, device)
         s = styles[i*num_groups + (grp % num_groups)]
         dev.setStyle((sem, layer), s)
@@ -519,27 +572,8 @@ class DeviceManager(object):
       dev.close()
     except Exception:
       pass
-    try:
-      # let an output device save its styles
-      dev.postCloseOutput(self.settings_manager)
-    except AttributeError:
-      pass
-    # @todo: PP: allow input devices to save state too
-    try:
-      self.out_devs.remove(dev)
-      log.debug('removed output device %s', dev.getName())
-    except ValueError:
-      pass
-    # @todo: PP: support unloading of input devices
-    try:
-      self.in_devs.remove(dev)
-      log.debug('removed input device %s', dev.getName())
-    except ValueError:
-      pass
-    if self.out_devs:
-      self.def_output = self.out_devs[0]
-    if self.in_devs:
-      self.def_input = self.in_devs[0]
+    self._unregisterOutputDevice(dev)
+    self._unregisterInputDevice(dev)
 
   def addMonitors(self, *monitors):
     '''
@@ -559,36 +593,19 @@ class DeviceManager(object):
     @rtype: 2-tuple L{AEMonitor.MonitorCollection}
     '''
     return self.in_mons, self.out_mons
-
-  def setDefaultOutput(self, dev):
+  
+  def getFirstOutput(self):
     '''
-    Sets the referenced output device as the default output device. When the
-    referenced device is not registered, or does not implement the
-    L{AEOutput} interface, this does nothing. 
-
-    @param dev: The input device
-    @type dev: L{AEOutput}
-    @raise NotImplementedError: When the provided device does not implement
-        L{AEOutput.AEOutput}
-    '''
-    if dev in self.out_devs:
-      self.def_output = dev
-    elif implements(dev, AEOutput.AEOutput):
-      if self.registerDevice(dev):
-        self.def_output = dev
-    else:
-      raise NotImplementedError('Not an output device')
-
-  def getDefaultOutput(self):
-    '''
-    Returns the current default output device or None when no output devices
-    have been registered.
-
-    @return: The current default output device
+    Gets the first L{AEOutput} device to successfully load.
+    
+    @return: First loaded output device or None if no output is available
     @rtype: L{AEOutput}
     '''
-    return self.def_output
-  
+    try:
+      return self.out_devs[0]
+    except IndexError:
+      return None
+
   def getOutputByName(self, name):
     '''
     Gets the L{AEOutput} device registered under the given name.
@@ -602,35 +619,20 @@ class DeviceManager(object):
       if dev.getClassName() == name:
         return dev
     return None
-
-  def setDefaultInput(self, dev):
+  
+  def getOutputByCaps(self, caps):
     '''
-    Sets the referenced input device as the default input device. When the
-    referenced device is not registered, or does not implement the
-    L{AEInput} interface, this does nothing. 
-
-    @param dev: The input device
-    @type dev: L{AEInput}
-    @raise NotImplementedError: When the provided device does not implement
-        L{AEInput.AEInput}
+    Gets the L{AEOutput} device registered with the given capabilities.
+    
+    @param caps: Desired capabilities of the output device.
+    @type caps: list of string
+    @return: Output device or None if caps could not be satisfied
+    @rtype: L{AEOutput}
     '''
-    if dev in self.in_devs:
-      self.def_input = dev
-    elif implements(dev, AEInput.AEInput):
-      if self.register(dev):
-        self.def_input = dev
-    else:
-      raise NotImplementedError('Not an input device')
-
-  def getDefaultInput(self):
-    '''
-    Returns the current default input device or None when no input devices
-    have been registered.
-
-    @return: The current default input device
-    @rtype: L{AEInput}
-    '''
-    return self.def_input
+    for dev in self.out_devs:
+      if set(dev.getCapabilities()).issuperset(caps):
+        return dev
+    return None
   
   def getInputByName(self, name):
     '''
@@ -645,8 +647,22 @@ class DeviceManager(object):
       if dev.getClassName() == name:
         return dev
     return None
+
+  def getInputByCaps(self, caps):
+    '''
+    Gets the L{AEInput} device registered with the given capabilities.
+    
+    @param caps: Desired capabilities of the input device.
+    @type caps: list of string
+    @return: Output device or None if caps could not be satisfied
+    @rtype: L{AEOutput}
+    '''
+    for dev in self.in_devs:
+      if set(dev.getCapabilities()).issuperset(caps):
+        return dev
+    return None
   
-  def _onGesture(self, gesture, timestamp):
+  def _onGesture(self, gesture, timestamp, **kwargs):
     '''    
     Creates an L{AEEvent} indicating the given gesture was found on a
     registered input device. When executed, the L{AEEvent} will notify the
@@ -657,19 +673,22 @@ class DeviceManager(object):
     @type gesture: L{AEInput.Gesture}
     @param timestamp: Time at which the event occurred
     @type timestamp: float
+    @param kwargs: Additional keyword arguments to pass to the L{Task}
+    @type kwargs: dictionary
     '''
-    event = AEEvent.InputGesture(gesture, timestamp)
+    event = AEEvent.InputGesture(gesture, timestamp, **kwargs)
     self.event_manager.postEvents(event)
     # notify monitors
-    self.in_mons.show(event=InputEvent.GESTURE, dev=gesture.getDevice(), 
-                      gesture=gesture)
+    self.in_mons.show(InputEvent(AEConstants.CMD_GESTURE), value=gesture, 
+                      dev=gesture.getDevice())
     
   def _onIndex(self, dev, index):
     '''
     @todo: PP: handle index callback markers
     '''
+    pass
     # notify monitors
-    self.out_mons.show(event=OutputEvent.MARKER)
+    #self.out_mons.show(event=OutputEvent.MARKER)
     
   def getDevices(self):
     '''
@@ -687,14 +706,12 @@ class DeviceManager(object):
     @param sem: The device semantic from which to get the style, or None to
       get the device's default style.
     @type sem: integer 
-    @param layer: Layer on which the event occurred, one of 
-      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     @return: the L{AEOutput.Style} for the given semantic of a device
     @rtype: L{AEOutput.Style}
     @raise KeyError: When the semantic or layer is invalid
     '''
-    dev = dev or self.def_output
     if dev is None: return None
     if sem is None or layer is None:
       return dev.getDefaultStyle()
@@ -704,66 +721,37 @@ class DeviceEvent(object):
   '''
   Base type for indicating IO events to monitors such that a monitor may 
   select whether to monitor input, output, both, or neither based on this
-  type and its subtypes. An L{AEMonitor} may query a subclass of this class
-  for its event types and their names in order to construct menus allowing the
-  user to select which events should be logged.
+  type and its subtypes.
   
-  @cvar default_types: List of device events that should be monitored by 
-    default
-  @type default_types: list of string
-  @cvar all_types: List of all device events
-  @type all_types: list of string
+  The L{cmd} attribute of an instance of this class should be considered 
+  public readable.
+  
+  @ivar cmd: Command constant for this event
+  @type cmd: object
   '''
-  default_types = []
-  all_types = []
-  
-  @classmethod
-  def getNames(cls):
+  def __init__(self, cmd):
     '''
-    @return: Names of all events
-    @rtype: list of string
+    Stores the command constant.
+    
+    @param cmd: Command constant for this event
+    @type cmd: object
     '''
-    return cls.all_types
-  
-  @classmethod
-  def getDefaults(cls):
-    '''
-    @return: Recommended names of events to be logged by default
-    @rtype: list of string
-    '''
-    return cls.default_types
-
+    self.cmd = cmd
+    
 class InputEvent(DeviceEvent):
-  '''
-  Subtype defining constants for input events.
+  '''Event on an input device, wrapped for display by a L{AEMonitor}.'''
+  def getName(self):
+    '''
+    @return: Human readable name for the command
+    @rtype: string
+    '''
+    return AEConstants.INPUT_COMMAND_NAMES[self.cmd]
   
-  @cvar GESTURE: Gesture was seen on an input device
-  @type GESTURE: string
-  '''
-  GESTURE = 'gesture'
-  default_types = []
-  all_types = [GESTURE]
-
 class OutputEvent(DeviceEvent):
-  '''
-  Subtype defining constants for output events.
-  
-  @cvar STRING: String was sent to an output device
-  @type STRING: string
-  @cvar FILENAME: Filename was sent to an output device
-  @type FILENAME: string
-  @cvar TALK: Talk was sent to an output device
-  @type TALK: string
-  @cvar INDEX: Index was sent to an output device
-  @type INDEX: string
-  @cvar MARKER: Marker was encountered during output
-  @type MARKER: string
-  '''
-  STRING = 'string'
-  TALK = 'talk'
-  INDEX = 'index'
-  FILENAME = 'filename'
-  MARKER = 'marker'
-  STOP = 'stop'
-  default_types = [STRING, TALK, FILENAME, STOP]
-  all_types = [STRING, TALK, FILENAME, MARKER, STOP]
+  '''Event on an output device, wrapped for display by a L{AEMonitor}.'''
+  def getName(self):
+    '''
+    @return: Human readable name for the command
+    @rtype: string
+    '''
+    return AEConstants.OUTPUT_COMMAND_NAMES[self.cmd]
