@@ -2,18 +2,21 @@
 Defines a class representing a multichannel audio output device. Uses pyibmtts
 and pySonicEx for speech synthesis and mixing respectively.
 
+@var UTF16_LANGS: All supported languages that accept UTF-16 encoded strings 
+  instead of CP1252 strings.
+@type UTF16_LANGS: tuple
 @var BUFFER_SIZE: Size of the speech buffer for each channel
 @type BUFFER_SIZE: integer
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import AEOutput
 import pySonicEx
@@ -23,68 +26,61 @@ from i18n import _
 
 __uie__ = dict(kind='device')
 
+UTF16_LANGS = (pyibmtts.MandarinChinese, pyibmtts.TaiwaneseMandarin, 
+               pyibmtts.StandardJapanese, pyibmtts.StandardKorean, 
+               pyibmtts.StandardCantonese)
 BUFFER_SIZE = 2**16
 
-class CliqueAudioStyle(AEOutput.StyleFlyweight):
+class CliqueAudioStyle(AEOutput.AudioStyle):
   '''
-  Overrides the base L{AEOutput.Style.StyleFlyweight} class to provide style
-  settings relative to the default style for this device.
-  '''
-  ADDITIVE = ('Pitch', 'Rate', 'Volume')
-  Rate = 0
-  Volume = 0
-  Pitch = 0
-  
-  def getSettings(self):
-    '''
-    Gets configurable relative settings affecting output from this device for
-    particular types of information.
-    
-    @todo: PP: support in a later version
-    
-    @return: Group of all configurable settings per semantic information type
-    @rtype: L{AEState.Setting.Group}
-    '''
-    raise NotImplementedError
-
-class CliqueAudioStyleDefault(AEOutput.AudioStyleDefault):
-  '''
-  Overrides the base L{AEOutput.Style.AudioStyleDefault} class, filling in 
+  Overrides the base L{AEOutput.Style.AudioStyle} class, filling in 
   fields for supported style properties with their appropriate values.
   '''
-  MaxVoice = 8
-  MinVoice = 1
-  MaxVolume = 65535
-  MinVolume = 1
-  Volume = 58982 # about 90%
-  MinPitch = 40
-  MaxPitch = 442
-  Pitch = 220
-  MaxRate = 1297
-  MinRate = 176
-  Rate = 300
-  Position = (0, 0, 0)
-  MaxChannel = 8
-  MinChannel = 0
-  Channel = 0
-  Voice = 1
-  
-  def getSettings(self):
+  def init(self, device):
+    self.newEnum('Voice', 1, _('Voice'),
+                 {_('Adult Male 1') : 1,
+                  _('Adult Female 1'): 2,
+                  _('Child 1') : 3,
+                  _('Adult Male 2') : 4,
+                  _('Adult Male 3') : 5,
+                  _('Adult Female 2') : 6,
+                  _('Elderly Female 1') : 7,
+                  _('Elderly Male 1') : 8},
+                 _('Name of the active voice'))
+    self.newRelRange('Pitch', 220, _('Pitch'), 40, 442, 0,
+                  _('Baseline voice pitch in Hertz'))
+    self.newRelRange('Rate', 250, _('Rate'), 176, 1297, 0,
+                  _('Speech rate in words per minute'))
+    self.newRelPercent('Volume', 58982, _('Volume'), 1, 65535, 0,
+                    _('Speech volume as a percentage'))
+    self.newNumeric('Channel', 0, _('Channel'), 0, 10, 0,
+                    _('Concurrent channel number'))
+    self.newNumeric('PosX', 0, _('Position X'), -10, 10, 1,
+                    _('Position of the source left-to-right'))
+    self.newNumeric('PosY', 0, _('Position Y'), -10, 10, 1,
+                    _('Position of the source up-and-down'))
+    self.newNumeric('PosZ', 0, _('Position Z'), -10, 10, 1,
+                    _('Position of the source front-to-back'))
+    
+  def getPosition(self):
+    '''
+    Convenience method for getting position as a 3-tuple.
+    
+    @return: x,y,z coordinate
+    @rtype: 3-tuple of float
+    '''
+    return self.PosX, self.PosY, self.PosZ
+
+  def getGroups(self):
     '''
     Gets configurable absolute settings affecting all output from this device.
     
     @return: Group of all configurable settings
     @rtype: L{AEState.Setting.Group}
     '''
-    g = self._newGroup()
-    a = g.newGroup(_('Speech'))
-    a.newRange('Pitch', _('Pitch'), self.MinPitch, self.MaxPitch, 0,
-               _('Baseline voice pitch in Hertz'))
-    a.newRange('Rate', _('Rate'), self.MinRate, self.MaxRate, 0,
-               _('Speech rate in words per minute'))
-    a.newPercent('Volume', _('Volume'), self.MinVolume, self.MaxVolume, 0,
-                 _('Speech volume as a percentage'))
-    #a.newNumeric('Channel', _('Channel'), self.MinChannel, self.MaxChannel, 0)
+    g = self.newGroup()
+    s = g.newGroup(_('Speech'))
+    s.extend(['Pitch', 'Rate', 'Volume'])
     # generate a group for standard word parsing settings
     self._newWordGroup(g)
     return g
@@ -99,8 +95,7 @@ class CliqueAudio(AEOutput.Audio):
   @type channels: dictionary of integer : L{Channel}
   '''
   USE_THREAD = False
-  DEFAULT_STYLE = CliqueAudioStyleDefault
-  FLYWEIGHT_STYLE = CliqueAudioStyle
+  STYLE = CliqueAudioStyle
   
   def _createNewChannel(self, i):
     '''
@@ -115,25 +110,6 @@ class CliqueAudio(AEOutput.Audio):
     ch = SpeechChannel(self.mixer), SoundChannel(self.mixer)
     self.channels[i] = ch
     return ch
-  
-  def _localize(self, text):
-    '''
-    Converts the given unicode text to latin-1, the encoding expected by the
-    IBM TTS runtime.
-
-    @todo: PP: Temporary kludge to convert all text to Latin-1 as IBM TTS is
-      expecting that encoding instead of UTF-8. Should look at the style 
-      language attribute in the future to do the correct encoding.
-    @param text: Text to localize
-    @type text: unicode
-    @return: Localized string
-    @rtype: string
-    '''
-    try:
-      # kludge!
-      return text.encode('latin-1', 'ignore')
-    except (TypeError, UnicodeDecodeError), e:
-      return text
  
   def init(self):
     '''
@@ -176,16 +152,16 @@ class CliqueAudio(AEOutput.Audio):
       for i in xrange(0, num_groups):
         # use CliqueAudioStyle for flyweights, not CliqueAudioStyleDefault
         s = CliqueAudioStyle(self.default_style)
+        s.init(self)
         # get a unique voice within this set
-        v_num = (i+1) % self.default_style.MaxVoice
+        v_num = (i % 8) + 1
         v = sch.tts.getVoice(v_num)
         s.Voice = v_num
         # set values relative to the default
-        s.Rate = 0
-        s.Pitch = v.pitchBaseline-self.default_style.Pitch
-        s.Volume = 0
+        s.Pitch = v.pitchBaseline
         # set to the position and channel of this layer
-        s.Position = (0.5*x, 0.5*y, 0)
+        s.PosX = 0.5*x
+        s.PosY = 0.5*y
         s.Channel = l
         styles.append(s)
     return styles
@@ -226,7 +202,7 @@ class CliqueAudio(AEOutput.Audio):
     except KeyError:
       ch = self._createNewChannel(style.Channel)
     # have the channel handle the text and style
-    ch[0].put(self._localize(text), style.copy())
+    ch[0].put(text, style.copy())
     
   def sendFilename(self, fn, style):
     '''
@@ -462,7 +438,7 @@ class SoundChannel(Channel):
       elif msg.cmd == msg.FILE:
         self.buffer.append(msg)
         # only keep track of the last requested position
-        self.last_pos = msg.style.Position
+        self.last_pos = msg.style.getPosition()
       elif msg.cmd == msg.TALK:
         for msg in self.buffer:
           # create the sound object for the synthed PCM data
@@ -522,6 +498,25 @@ class SpeechChannel(Channel):
     # settings used by pySonicEx
     self.mode = pySonicEx.OPENRAW|pySonicEx.OPENMEMORY|pySonicEx.ThreeD
     self.start()
+    
+  def _localize(self, text, style):
+    '''
+    Converts the given unicode text to CP1252 or UTF-16 depending on the 
+    language flag in the style.
+
+    @note: Larry says IBM TTS expects all Asian language strings to be encoded
+      in UCS-2, predecessor of UTF-16, and all others in CP1252.
+    @param text: Text to localize
+    @type text: unicode
+    @param style: Style object indicating the language and locale
+    @type style: L{AEOutput.Style}
+    @return: Localized string
+    @rtype: string
+    '''
+    if self.tts.languageDialect in UTF16_LANGS:
+      return text.encode('utf-16', 'ignore')
+    else:
+      return text.encode('cp1252', 'ignore')
         
   def put(self, text, style):
     '''
@@ -573,13 +568,13 @@ class SpeechChannel(Channel):
         self.want_stop = False
       elif msg.cmd == msg.TEXT:
         # apply a new style
-        if msg.style.isDirty() or self.last_style != msg.style:
+        if msg.style and (msg.style.isDirty() or self.last_style != msg.style):
           # talk immediately if a pySonicEx style property changed
           talk = self._applyStyle(msg.style)
           self.last_style = msg.style
         # queue new text
         if msg.text is not None:
-          self.tts.addText(msg.text)
+          self.tts.addText(self._localize(msg.text, msg.style))
       elif (msg.cmd == msg.TALK or talk == True) and self.tts.speaking():
         # synthesize speech synchrnously to the buffer
         self.tts.synthSync()
@@ -632,27 +627,27 @@ class SpeechChannel(Channel):
     v = self.tts.setActiveVoice(v)
     # restore voice independent parameters
     try:
-      v.speed = min(max(style.Rate, style.MinRate), style.MaxRate)
+      v.speed = style.Rate
     except pyibmtts.ECIException:
       pass
     try:
-      v.volume = min(max(style.Volume, style.MinVolume), style.MaxVolume)
+      v.volume = style.Volume
     except pyibmtts.ECIException:
       pass
     try:
-      v.pitchBaseline = min(max(style.Pitch, style.MinPitch), style.MaxPitch)
+      v.pitchBaseline = style.Pitch
     except pyibmtts.ECIException:
       pass
     
     if self.last_pos is None:
       # if position was never specified, store it now but indicate it has not
       # changed
-      self.last_pos = style.Position
+      self.last_pos = style.getPosition()
       return False
-    elif self.last_pos != style.Position:
+    elif self.last_pos != style.getPosition():
       # if the new position differs from the previous, store it and indicate
       # a talk should be done implicitly to respect the new position
-      self.last_pos = style.Position
+      self.last_pos = style.getPosition()
       return True
     else:
       # otherwise, just return
