@@ -21,7 +21,7 @@ See the LSR man pages for help with using command line parameters.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -37,10 +37,10 @@ except ImportError:
   pass
 
 import optparse, os, signal, sys, subprocess, glob, re
-import UIRegistrar, LSRConstants, AEOutput, LSRLog
+import UIRegistrar, AEConstants, AEOutput, LSRLog
 from i18n import _
 from SettingsManager import SettingsManager
-from LSRConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
+from AEConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
      TEMPLATE_DIR, DEFAULT_ASSOCIATIONS, DEFAULT_PROFILE
 from LSRInterfaces import implements
 
@@ -48,10 +48,11 @@ from LSRInterfaces import implements
 wrap_rx=re.compile(u"([\u2e80-\uffff])", re.UNICODE)
 
 def wrap(text, width=79, encoding='utf-8', indent=4):
-  '''  
+  '''    
   A word-wrap function that preserves existing line breaks and most spaces in
-  the text. Expects that existing line breaks are posix newlines (\n). Recipe
-  by Junyong Pan from the Python Cookbook.
+  the text. Expects that existing line breaks are posix newlines. Recipe by
+  Junyong Pan from the Python Cookbook:
+  
   U{http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/358117}
   
   @param text: Text to wrap
@@ -75,8 +76,8 @@ def welcome():
   Prints copyright, license, and version info.
   '''
   print '%s ver. %s rev. %s' % \
-        (LSRConstants.NAME, LSRConstants.PROG_VERSION, LSRConstants.PROG_DATE)
-  print LSRConstants.COPYRIGHT
+        (AEConstants.NAME, AEConstants.PROG_VERSION, AEConstants.PROG_DATE)
+  print AEConstants.COPYRIGHT
   print
   
 def confirmRemove(name):
@@ -92,7 +93,7 @@ def confirmRemove(name):
   @return: Confirm (True) or cancel (False)?
   @rtype: boolean
   '''
-  ch = raw_input(LSRConstants.MSG_CONFIRM_REMOVE % name)
+  ch = raw_input(AEConstants.MSG_CONFIRM_REMOVE % name)
   return ch == 'y'
 
 def install(options):
@@ -213,6 +214,7 @@ def generate(options):
   rv = install(options)
   # associate the UIE with the given profiles
   options.action_value = name
+  options.profile = options.profile or AEConstants.GENERATE_PROFILES
   return associate(options)
   
 def createProfile(options):
@@ -263,10 +265,22 @@ def removeProfile(options):
     return _('Cancelled removal of profile %s') % options.action_value
   sm.deleteProfile()
   return _('Removed profile %s') % options.action_value
+
+def exportPath(options):
+  '''
+  Gets the path to the package containing this file. Essentially, this is
+  the path to the LSR package on disk.
+  
+  @param options: Options seen on the command line
+  @type options: optparse.Values
+  @return: Description of the action
+  @rtype: string
+  '''
+  return os.path.abspath(os.path.normpath(os.path.dirname(__file__)))
   
 def say(options):
   '''
-  Uses the default L{AEOutput} device to output a string.
+  Uses the first available L{AEOutput} device to output a string.
   
   @param options: Options seen on the command line
   @type options: optparse.Values
@@ -438,10 +452,9 @@ def getAction(option, opt_str, value, parser):
 def configA11y():
   '''
   Runs gconf to check if desktop accessibility support is enabled or not. If 
-  not, enables it and asks the user to logout. Also enables accessibility for
-  this application by setting GTK_MODULES.
+  not, enables it and asks the user to logout. 
   
-  @note: GNOME dependent
+  @note: Current implementation is AT-SPI dependent
   @raise ValueError: When accessibility support is not enabled
   '''
   # make sure desktop accessibility is enabled
@@ -452,7 +465,28 @@ def configA11y():
   cl = gconf.client_get_default()
   if not cl.get_bool('/desktop/gnome/interface/accessibility'):
     cl.set_bool('/desktop/gnome/interface/accessibility', True)
-    raise ValueError(LSRConstants.HELP_A11Y)
+    raise ValueError(AEConstants.HELP_A11Y)
+  
+def configGUI():
+  '''  
+  Configures the GUI toolkit with a default icon list for all LSR top level 
+  windows. Also enables accessibility for this application.
+
+  @note: Current implementation is gtk/gail dependent
+  '''
+  import pygtk
+  pygtk.require('2.0')
+  import gtk
+  # load all sizes of icons
+  it = gtk.IconTheme()
+  try:
+    icons = [it.load_icon(AEConstants.PROG_NAME, size, gtk.ICON_LOOKUP_NO_SVG) 
+             for size in AEConstants.ICON_SIZES]
+  except Exception:
+    # ignore errors, and just don't use the icon
+    pass
+  else:
+    gtk.window_set_default_icon_list(*icons)
   # add GTK_MODULES to the environment so all GUIs are accessible
   os.environ['GTK_MODULES'] = 'gail:atk-bridge'
 
@@ -468,62 +502,64 @@ def parseOptions():
   op = optparse.OptionParser()
   # add all the command line options to the parser
   op.add_option('-g', '--generate', action='callback', callback=getAction,
-                help=LSRConstants.HELP_GENERATE, default=generate, type='str',
+                help=AEConstants.HELP_GENERATE, default=generate, type='str',
                 nargs=2)
   op.add_option('-i', '--install', action='callback', callback=getAction,
-                help=LSRConstants.HELP_INSTALL, default=install, type='str',
+                help=AEConstants.HELP_INSTALL, default=install, type='str',
                 nargs=1)
   op.add_option('-u', '--uninstall', action='callback', callback=getAction,
-                help=LSRConstants.HELP_UNINSTALL, default=uninstall,type='str',
+                help=AEConstants.HELP_UNINSTALL, default=uninstall,type='str',
                 nargs=1)
   op.add_option('-a', '--associate', action='callback', callback=getAction,
-                help=LSRConstants.HELP_ASSOCIATE, default=associate,type='str',
+                help=AEConstants.HELP_ASSOCIATE, default=associate,type='str',
                 nargs=1)
   op.add_option('-d', '--disassociate', action='callback', callback=getAction,
-                help=LSRConstants.HELP_DISASSOCIATE, default=disassociate,
+                help=AEConstants.HELP_DISASSOCIATE, default=disassociate,
                 type='str', nargs=1)
   op.add_option('-c', '--create-profile', action='callback',callback=getAction,
-                help=LSRConstants.HELP_CREATE_PROFILE, default=createProfile,
+                help=AEConstants.HELP_CREATE_PROFILE, default=createProfile,
                 type='str', nargs=1)
   op.add_option('-r', '--remove-profile', action='callback',callback=getAction,
-                help=LSRConstants.HELP_REMOVE_PROFILE, default=removeProfile,
+                help=AEConstants.HELP_REMOVE_PROFILE, default=removeProfile,
                 type='str', nargs=1)
   op.add_option('--dupe', '--duplicate-profile', action='callback', 
                 callback=getAction, default=duplicateProfile,
-                help=LSRConstants.HELP_DUPLICATE_PROFILE, type='str', nargs=2)
+                help=AEConstants.HELP_DUPLICATE_PROFILE, type='str', nargs=2)
   op.add_option('-s', '--show', action='callback', callback=getAction,
-                default=show, help=LSRConstants.HELP_SHOW)
+                default=show, help=AEConstants.HELP_SHOW)
   op.add_option('-p', '--profile', action='append', dest='profile',
-                help=LSRConstants.HELP_PROFILE)
+                help=AEConstants.HELP_PROFILE)
   op.add_option('-y', '--say', action='callback', callback=getAction,
-                help=LSRConstants.HELP_SAY, default=say, type='str')
+                help=AEConstants.HELP_SAY, default=say, type='str')
   op.add_option('--index', dest='uie_index', action='store', type='int',
-                help=LSRConstants.HELP_INDEX)
+                help=AEConstants.HELP_INDEX)
   op.add_option('--global', action='store_true', default=False,
-                dest='uie_global', help=LSRConstants.HELP_GLOBAL)
+                dest='uie_global', help=AEConstants.HELP_GLOBAL)
   op.add_option('--all-tiers', '--all-apps', action='store_true', 
                 default=False,
-                dest='uie_all_tiers', help=LSRConstants.HELP_ALL_TIERS)
+                dest='uie_all_tiers', help=AEConstants.HELP_ALL_TIERS)
   op.add_option('--tier', '--app', dest='uie_tier', action='store', type='str',
-                help=LSRConstants.HELP_TIER)
+                help=AEConstants.HELP_TIER)
   op.add_option('-l', '--log-level', dest='log_level', action='store',
                 choices=['none', 'debug', 'print', 'info', 'warning', 'error'],
-                help=LSRConstants.HELP_LOG_LEVEL % '[debug, print, info, ' \
+                help=AEConstants.HELP_LOG_LEVEL % '[debug, print, info, ' \
                 'warning, error, critical]')
   op.add_option('--log-channel', dest='log_channels', action='append',
-                help=LSRConstants.HELP_LOG_CHANNEL % '[Perk, Tier, ...]')
+                help=AEConstants.HELP_LOG_CHANNEL % '[Perk, Tier, ...]')
   op.add_option('--log-file', dest='log_file', action='store', type='str',
-                help=LSRConstants.HELP_LOG_FILENAME)
+                help=AEConstants.HELP_LOG_FILENAME)
   op.add_option('--log-conf', dest='log_conf', action='store', type='str',
-                help=LSRConstants.HELP_LOG_CONFIG)
+                help=AEConstants.HELP_LOG_CONFIG)
   op.add_option('--init-global', action='callback', callback=getAction, 
-                default=initGlobal, help=LSRConstants.HELP_INIT_GLOBAL)
+                default=initGlobal, help=AEConstants.HELP_INIT_GLOBAL)
   op.add_option('--init-profiles', action='callback', callback=getAction, 
-                default=initProfiles, help=LSRConstants.HELP_INIT_PROFILES)
+                default=initProfiles, help=AEConstants.HELP_INIT_PROFILES)
   op.add_option('--no-intro', action='store_true', default=False, 
-                dest='no_intro', help=LSRConstants.HELP_NO_INTRO)
+                dest='no_intro', help=AEConstants.HELP_NO_INTRO)
   op.add_option('--no-a11y', action='store_true', default=False, 
-                dest='no_a11y')
+                dest='no_a11y', help=AEConstants.HELP_NO_A11Y)
+  op.add_option('--export-path', action='callback', callback=getAction,
+                default=exportPath, help=AEConstants.HELP_EXPORT_PATH)
   (options, args) = op.parse_args()
   return options
 
@@ -537,8 +573,6 @@ def main():
   checks that a11y support is enabled in gconf. If not, enables it but exits
   with an error.
   '''
-  # show the welcome message
-  welcome()
   # parse the command line options
   options = parseOptions()
 
@@ -553,10 +587,16 @@ def main():
     # if not, run LSR
     mtd = run
 
+  if mtd is not exportPath:
+    # show the welcome message
+    welcome()
+
   if mtd is run:
     if not options.no_a11y:
       # make sure accessibility is enabled on the desktop and for LSR
       configA11y()
+    # configure GUI toolkit
+    configGUI()
     # set up the logging system
     LSRLog.configure(options.log_conf, options.log_level, options.log_channels,
                      options.log_file)
