@@ -4,7 +4,7 @@ handle them.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -12,11 +12,10 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-from AEEvent import LAYER_NAMES
-from LSRConstants import SEMANTIC_NAMES
+from AEConstants import LAYER_NAMES, SEMANTIC_NAMES
 from DeviceManager import InputEvent, OutputEvent, DeviceEvent
 from GTKEventDialog import GTKEventDialog
-import AEInput, AEOutput
+import AEInput, AEOutput, AEConstants
 from i18n import _
 
 __uie__ = dict(kind='monitor', profiles=['developer'])
@@ -102,13 +101,13 @@ class IOMonitor(GTKEventDialog):
       self.last_dev = name
       self._queueText('\n\n{device: %s}\n' % name)
 
-    if event.cmd == AEOutput.CMD_TALK:
+    if event.cmd == AEConstants.CMD_TALK:
       self._queueText(' {%s: %s} ' %(event.getName(), SEMANTIC_NAMES.get(sem)))
-    elif event.cmd == AEOutput.CMD_STOP:
+    elif event.cmd == AEConstants.CMD_STOP:
       self._queueText('{%s}\n' % event.getName())
-    elif event.cmd == AEOutput.CMD_STRING:
+    elif event.cmd == AEConstants.CMD_STRING:
         self._queueText(str(value))
-    elif event == AEOutput.CMD_FILENAME:
+    elif event == AEConstants.CMD_FILENAME:
         self._queueText('{%s: %s}' % (event.getName(), str(value)))
     else:
       self._queueText('{%s: %s}\n' % (event.getName(), str(value)))
