@@ -5,16 +5,16 @@ response to L{AEEvent}s and L{AEInput.Gesture}s.
 @author: Peter Parente
 @author: Pete Brunet
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import logging, weakref, os
-import Task, AEInput, AEEvent, LSRInterfaces, UIElement
+import logging, weakref, os, sys
+import Task, AEInput, AEEvent, LSRInterfaces, UIElement, AEConstants
 from i18n import _
 from AEState import AEState as PerkState
 
@@ -70,12 +70,23 @@ class Perk(Task.Tools.All, UIElement.UIE):
     L{AEInput.GestureList} sequence on some L{AEInput} device. The name should
     be hashed into the L{named_tasks} dictionary to fully resolve the L{Task}.
   @type commands: weakref.WeakValueDictionary of (GestureList : string)
-  @ivar state: Settings for this L{Perk}. Defaults to a dummy object that can
-    be used to store state information that is not configurable or 
-    persistable until registerState is called.
-  @type state: L{AEState.AEState}
-  @ivar choosers: L{Task}s to execute in response to a L{AEEvent.ChooserChange}
-  @type choosers: weakref.WeakValueDictionary of (Chooser : Task)
+  @ivar state: Settings for this L{Perk}. Defaults to an empty L{PerkState} 
+    object that can be used to store state information that is not 
+    configurable or persistable unless L{STATE} is specified.
+  @type state: L{PerkState}
+  @ivar out_caps: Capabilities set by L{setIdealOutput} as ideal for output
+    from this L{Perk}. These are used to "late bind" a device for use during
+    execution of this L{Perk} and its L{Task}s. Defaults to audio.
+  @type out_caps: list of string
+  @ivar before_chains: Mapping from target L{Task} name to a list of L{Task}
+    names which should be executed before the target
+  @type before_chains: dictionary of {string : list of string}
+  @ivar after_chains: Mapping from target L{Task} name to a list of L{Task}
+    names which should be executed after the target
+  @type after_chains: dictionary of {string : list of string}
+  @ivar around_chains: Mapping from target L{Task} name to one L{Task} name 
+    which should be executed instead of the target
+  @type around_chains: dictionary of {string : list of string}
   '''
   STATE = PerkState
   
@@ -93,9 +104,18 @@ class Perk(Task.Tools.All, UIElement.UIE):
     self.keyed_tasks = {}
     self.event_tasks = {}
     self.commands = {}
+    # default to audio output
+    self.out_caps = ['audio']
+    # define empty chains for tasks
+    self.before_chains = {}
+    self.after_chains = {}
+    self.around_chains = {}
+
     # instantiate an instance of the STATE class; it may be filled with data
     # when preInit is called with state already persisted to disk
     self.state = self.STATE()
+    # initialize state settings
+    self.state.init()
     
   def preInit(self, *args, **kwargs):
     '''
@@ -134,6 +154,30 @@ class Perk(Task.Tools.All, UIElement.UIE):
     self.keyed_tasks = {}
     self.event_tasks = {}
     self.commands = {}
+    
+  def setIdealOutput(self, *capabilities):
+    '''
+    Sets the ideal device capabilities to be used to locate a device for all 
+    output methods invoked from the caller's L{Perk} module, including the 
+    Perk class itself and all of its registered L{Task}s. The capabilities list
+    should include strings naming L{AEOutput} interfaces ("audio" and/or 
+    "braille") at present.
+    
+    @param capabilities: Names of capabilities required on the device
+    @type capabilities: list of string    
+    '''
+    self.out_caps = capabilities
+    
+  def getIdealOutput(self):
+    '''        
+    Gets the ideal device capabilities to locate a device to be used by all
+    output methods invoked from the caller's L{Perk} module, including the Perk
+    class itself and all of its registered L{Task}s.
+    
+    @return: Names of capabilities set as ideal for a default output device
+    @rtype: list of string
+    '''
+    return self.out_caps
   
   def getState(self):
     '''
@@ -143,7 +187,7 @@ class Perk(Task.Tools.All, UIElement.UIE):
     @rtype: L{AEState.AEState}
     '''
     return self.state
-  
+
   def _initTask(self, task):
     '''
     Initializes a L{Task} by calling L{Task.Task.preInit} and 
@@ -169,9 +213,9 @@ class Perk(Task.Tools.All, UIElement.UIE):
     self._initTask(task)
     self.keyed_tasks[tid] = task
     # use the access engine timer to notify our callback
-    self.acc_eng.addTimer(_postTimerEvent, interval*1000, interval, tid, aid,
+    self.acc_eng.addTimer(_postTimerEvent, interval, interval, tid, aid,
                           wtask, weman)
-    self.tier.addPerkRef(tid, self)
+    self.tier.addTaskRef(tid, self)
   
   def registerCommandTask(self, device, codes, name, propogate=False):
     '''
@@ -200,18 +244,23 @@ class Perk(Task.Tools.All, UIElement.UIE):
       # raise a value error if there is already a command registered for this
       # sequence of keys in this Tier
       raise ValueError(
-        'Command with action codes %s on device %s is already registered' % 
+        'Command already registered with action codes %s on device %s' % 
         (codes, device.getName()))
     # add key list to our dictionary of commands and store the task to execute
     # as its value
     self.commands[gl] = name
-    self.tier.addPerkRef(gl, self)
+    self.tier.addTaskRef(gl, self)
     if not propogate:
       # try to add a filter to the device, ignore if not supported
       try:
         device.addFilter(gl)
       except NotImplementedError:
         pass
+    # accept a KEY_CMD from the device, ignore if not supported  
+    try:
+      device.addKeyCmd(codes)
+    except NotImplementedError:
+      pass
       
   def registerChooserTask(self, chooser, task):
     '''
@@ -227,9 +276,8 @@ class Perk(Task.Tools.All, UIElement.UIE):
     if self.tier.getKeyedTask(cid) is not None:
       raise ValueError('Chooser Task already registered')
     self._initTask(task)
-    # the Task has already been initialized, so don't do it again
     self.keyed_tasks[cid] = task
-    self.tier.addPerkRef(cid, self)
+    self.tier.addTaskRef(cid, self)
     
   def registerNamedTask(self, task, name):
     '''
@@ -251,7 +299,7 @@ class Perk(Task.Tools.All, UIElement.UIE):
       # initialize the Task
       self._initTask(task)
       self.named_tasks[name] = task
-      self.tier.addPerkRef(name, self)
+      self.tier.addTaskRef(name, self)
     else:
       raise ValueError(name)
     
@@ -297,14 +345,19 @@ class Perk(Task.Tools.All, UIElement.UIE):
     '''
     # default to the focus layer if no layer is specified    
     default = not (focus or tier or background)
-    d = {AEEvent.LAYER_FOCUS : focus or default, AEEvent.LAYER_TIER : tier, 
-         AEEvent.LAYER_BACKGROUND : background}
+    d = {AEConstants.LAYER_FOCUS : focus or default, 
+         AEConstants.LAYER_TIER : tier, 
+         AEConstants.LAYER_BACKGROUND : background}
     # get the kind of AEEvent the Task 
     kind = task.getEventType()
     # initialize the Task
     self._initTask(task)
     # opt: inform the Tier of the kind of event that is desired by this Perk
-    self.tier.setEventInterest(kind, True)
+    try:
+      self.tier.setEventInterest(kind, True)
+    except KeyError:
+      # ignore unknown event types, could be internal
+      pass
     for layer, val in d.items():
       if val:
         # get the list of Task registered for this type and layer or insert an
@@ -322,9 +375,11 @@ class Perk(Task.Tools.All, UIElement.UIE):
     @raise KeyError: When the L{Task} is not registered
     '''
     tid = id(task)
-    task = self.keyed_tasks[tid]
     del self.keyed_tasks[tid]
-    self.tier.removePerkRef(tid)
+    # let the task finalize itself
+    task.close()
+    task.postClose()
+    self.tier.removeTaskRef(tid)
 
   def unregisterCommandTask(self, device, codes):
     '''
@@ -345,10 +400,16 @@ class Perk(Task.Tools.All, UIElement.UIE):
     # codes are the same
     gl = AEInput.GestureList(device, codes)
     del self.commands[gl]
-    self.tier.removePerkRef(gl)
+    self.tier.removeTaskRef(gl)
     # try to remove a filter from the device, ignore if not supported
     try:
       device.removeFilter(gl)
+    except (NotImplementedError, AttributeError):
+      pass
+    
+    # remove an accepted KEY_CMD from the device, ignore if not supported
+    try:
+      device.removeKeyCmd(codes)
     except NotImplementedError:
       pass
     
@@ -364,7 +425,13 @@ class Perk(Task.Tools.All, UIElement.UIE):
     cid = id(chooser)
     task = self.keyed_tasks[cid]
     del self.keyed_tasks[cid]
-    self.tier.removePerkRef(cid)
+    # A chooser task is an odd beast because
+    # when a chooser dialog is getting destroyed, we want to unhook the task
+    # that was handling notifications for it from the Perk. However, we want
+    # that task to stay initialized for the next time the chooser is shown. 
+    # task.close()
+    # task.postClose()
+    self.tier.removeTaskRef(cid)
 
   def unregisterNamedTask(self, name):
     '''
@@ -380,7 +447,7 @@ class Perk(Task.Tools.All, UIElement.UIE):
     # let the task finalize itself
     task.close()
     task.postClose()
-    self.tier.removePerkRef(name)
+    self.tier.removeTaskRef(name)
     
   def unregisterEventTask(self, task, focus=False, tier=False, 
                           background=False):
@@ -407,8 +474,8 @@ class Perk(Task.Tools.All, UIElement.UIE):
     '''
     # default to the focus layer if no layer is specified    
     default = not (focus or tier or background)
-    d = {AEEvent.LAYER_FOCUS : focus or default, AEEvent.LAYER_TIER : tier, 
-         AEEvent.LAYER_BACKGROUND : background}
+    d = {AEConstants.LAYER_FOCUS : focus or default, AEConstants.LAYER_TIER : tier, 
+         AEConstants.LAYER_BACKGROUND : background}
     # get the task type
     kind = task.getEventType()
     # let the Task finalize itself
@@ -416,7 +483,11 @@ class Perk(Task.Tools.All, UIElement.UIE):
     task.postClose()
     # opt: inform the Tier of the kind of event that is no longer
     # desired by this Perk
-    self.tier.setEventInterest(kind, False)
+    try:
+      self.tier.setEventInterest(kind, False)
+    except KeyError:
+      # ignore unknown event types, could be internal
+      pass
     for layer, val in d.items():
       if val:
         # get the list of Task registered for this type and layer or insert an
@@ -483,11 +554,128 @@ class Perk(Task.Tools.All, UIElement.UIE):
     
     @param event_type: Desired type of L{AEEvent}
     @type event_type: L{AEEvent} class
-    @param task_layer: Layer on which the desired L{Task}s are registered, one 
-      of L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param task_layer: Layer on which the desired L{Task}s are registered
     @type task_layer: integer
     @return: List of all L{Task}s that handle the given event type on the given
       layer in this L{Perk}
     @rtype: list of L{Task.Base.Task}
     '''
     return list(self.event_tasks.get((event_type, task_layer), []))
+  
+  def getChainSegment(self, target, link):
+    '''
+    Gets a copy of the segment of the L{Task} chain for target specified by the
+    link type. A copy is returned to avoid iteration errors if the list changes
+    size during execution of the L{Task}s in the chain.
+    
+    @param target: Name of the L{Task} to link to
+    @type target: string
+    @param link: One of the CHAIN constants in L{AEConstants.Tools} 
+    @type link: integer
+    @raise KeyError: When there is no segment for the given target and link
+      type
+    '''
+    if link == AEConstants.CHAIN_BEFORE:
+      return self.before_chains.get(target, [])[:]
+    elif link == AEConstants.CHAIN_AFTER:
+      return self.after_chains.get(target, [])[:]
+    elif link == AEConstants.CHAIN_AROUND:
+      return self.around_chains
+  
+  def linkTaskToChain(self, name, link, target):
+    '''
+    Links a L{Task} to the one named in target. The L{Task} will be added 
+    either before, after, or around depending on the value of link. Does not
+    allow a L{Task} to link to itself (i.e. name cannot equal target).
+    
+    Invokes L{Tier.Tier.addChainRef} to add a reference to this L{Perk} to the
+    L{Tier} which will later have to traverse the chain across L{Perk}s.
+
+    @param name: Name of the L{Task} to link
+    @type name: string
+    @param link: One of the CHAIN constants in L{AEConstants.Tools} 
+    @type link: integer
+    @param target: Name of the L{Task} to link to
+    @type target: string
+    @raise ValueError: When the link type is unknown
+    @raise AssertionError: When the name equals the target
+    '''
+    assert name != target
+    if link == AEConstants.CHAIN_BEFORE:
+      self.before_chains.setdefault(target, []).append(name)
+    elif link == AEConstants.CHAIN_AFTER:
+      self.after_chains.setdefault(target, []).append(name)
+    elif link == AEConstants.CHAIN_AROUND:
+      self.around_chains[target] = name
+    else:
+      raise ValueError(link)
+    # add a reference to this Perk to the Tier
+    self.tier.addChainRef(target, self)
+      
+  def unlinkTaskFromChain(self, name, link, target):
+    '''
+    Unlinks a L{Task} from the one named in target. The L{Task} will be
+    unlinked either before, after, or around depending on the value of link. 
+    If link is None, the L{Task} will be unlinked from all cases if
+    possible.
+    
+    Invokes L{Tier.Tier.removeChainRef} to delete a reference to this L{Perk}
+    to the L{Tier} which will later have to traverse the chain across L{Perk}s.
+    
+    @todo: PP: call tier.removeChainRef when there are no more links for the
+      target in this Perk
+
+    @param name: Name of the L{Task} to link
+    @type name: string
+    @param link: One of the CHAIN constants in L{AEConstants.Tools} 
+    @type link: integer
+    @param target: Name of the L{Task} to unlink from
+    @type target: string
+    @raise ValueError: When the named L{Task} is not unlinked at least once or
+      the link type is unknown
+    @raise KeyError: When the target L{Task} does not have a chain segement in
+      this L{Perk} for the given link type
+    '''
+    if link is not None:
+      if link == AEConstants.CHAIN_BEFORE:
+        self.before_chains[target].remove(name)
+      elif link == AEConstants.CHAIN_AFTER:
+        self.after_chains[target].remove(name)
+      elif link == AEConstants.CHAIN_AROUND:
+        # don't unlink around unless it matches
+        if self.around_chains[target] == name:
+          del self.around_chains[target]
+          self.tier.removeChainRef(target, self)
+        else:
+          raise ValueError(name)
+      else:
+        raise ValueError(link)
+      # remove a reference to this Perk from the Tier
+      #self.tier.removeChainRef(target, self)
+    else:
+      # track how many removals we make
+      count = 0
+      # try removing from all chain segments
+      try:
+        self.before_chains[target].remove(name)
+        count += 1
+      except (KeyError, ValueError):
+        pass
+      try:
+        self.after_chains[target].remove(name)
+        count += 1
+      except (KeyError, ValueError):
+        pass
+      try:
+        if self.around_chains[target] == name:
+          self.around_chains[target] = None
+          count += 1
+      except (KeyError, ValueError):
+        pass
+
+      if count == 0:
+        # error if we didn't remove anything
+        raise ValueError
+#      else:
+        # remove a reference to this Perk from the Tier
+        #self.tier.removeChainRef(target, self)
\ No newline at end of file
