@@ -3,7 +3,7 @@ Defines a L{Task} to execute when a L{AEEvent.InputGesture} occurs.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -11,15 +11,16 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import Base, Constants
+import Base
 
 class InputTask(Base.Task):
   '''
-  Executes when an input gesture occurs.
-
+  Executes when an input gesture occurs. Does nothing different than the
+  base class at present.
+  
   This class registers its name and whether it should be monitored by default
-  in an L{AEMonitor} using the L{Constants.registerTaskType}
-  function when this module is first imported. The L{AEMonitor}
-  can use this information to build its menus.  
+  in an L{AEMonitor} using the L{Base.registerTaskType} function when this
+  module is first imported. The L{AEMonitor} can use this information to build
+  its menus.
   '''
-  Constants.registerTaskType('InputTask', False)
+  Base.registerTaskType('InputTask', True)
\ No newline at end of file
