@@ -5,17 +5,18 @@ application.
 @author: Peter Parente
 @author: Pete Brunet
 @author: Larry Weiss
+@author: Scott Haeger
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import logging, weakref
-import AEState, AEInput, Perk, Task
+import AEState, AEInput, Perk, Task, AEConstants
 from POR import POR
 
 log = logging.getLogger('Tier')
@@ -44,6 +45,20 @@ class Tier(object):
   @type name: string
   @ivar aid: ID uniquely identifying this L{Tier} from all other L{Tier}s
   @type aid: opaque
+  @ivar last_key: Stores the key code, key sym, and modifiers for the last key 
+    pressed
+  @type last_key: 3-tuple of integer, string, integer
+  @ivar perk_refs: References to L{Perk}s keyed by task names, commands, 
+    identifiers, etc. Many to one mapping. See L{addTaskRef} and 
+    L{removeTaskRef}. 
+  @type perk_refs: weakref.WeakValueDictionary
+  @ivar chain_refs: References to L{Perk}s keyed by task names. Many to many
+    mapping. See L{addChainRef} and L{removeChainRef}.
+  @type chain_refs: dictionary
+  @ivar temp_data: Arbitrary name/value pairs stored by L{Task}s executing in
+    this L{Tier} in response to a single event. The data is persistent until 
+    the event has been handled by all L{Task}s at which time it is cleared.
+  @type temp_data: dictionary
   '''
   def __init__(self, tier_manager, name, aid, por):
     '''
@@ -68,8 +83,11 @@ class Tier(object):
     self.name = name
     self.aid = aid
     self.perk_refs = weakref.WeakValueDictionary()
+    self.chain_refs = {}
     self.pointer_por = por
     self.focus_por = None
+    self.last_key = (None, None)
+    self.temp_data = {}
 
   def close(self):
     '''
@@ -103,8 +121,78 @@ class Tier(object):
     @rtype: L{POR}
     '''
     return self.focus_por
+  
+  def getLastKey(self):
+    '''    
+    Gets the last key code, key sym, and key modifiers. Useful for dealing with
+    applications that do a poor job of synthesizing caret and text events.
+    
+    @return: The key code, key sym, and modifiers for the last key pressed
+    @rtype: 3-tuple of integer, string, integer
+    '''
+    return self.last_key
+  
+  def getTempVal(self, name):
+    '''
+    Gets data from L{temp_data}.
+    
+    @param name: Name of the stored data
+    @type name: immutable
+    @return: Value stored under the given name
+    @rtype: object
+    @raise KeyError: When no data is stored under the given name
+    '''
+    return self.temp_data[name]
+  
+  def setTempVal(self, name, value):
+    '''
+    Stores data for the duration of the execution of an L{AEEvent}.
+    
+    @param name: Name to associate with the value
+    @type name: immutable
+    @param value: Any value to store
+    @type value: object
+    '''
+    self.temp_data[name] = value
+    
+  def clearChainData(self):
+    '''
+    Clears all stored L{temp_data}.
+    '''
+    self.temp_data = {}
+  
+  def addChainRef(self, target, perk):
+    '''
+    Adds a weak reference to a L{Perk} as one to check for L{Task}s which are 
+    part of a the chain for the target. Optimization for the L{manageNamedTask}
+    method.
 
-  def addPerkRef(self, key, perk):
+    @param target: Name of the L{Task} to link to
+    @type target: string
+    @param perk: Perk have at least one link in the chain for the target
+    @type perk: L{Perk} 
+    '''
+    chain = self.chain_refs.setdefault(target, weakref.WeakKeyDictionary())
+    chain[perk] = None
+     
+  def removeChainRef(self, target, perk):
+    '''
+    Removes a weak reference to a L{Perk} as one to no longer check for 
+    L{Task}s which are part of the chain for the target. Optimization for the 
+    L{manageNamedTask} method.
+    
+    @param target: Name of the L{Task} to unlink from
+    @type target: string
+    @param perk: Perk have at least one link in the chain for the target
+    @type perk: L{Perk}
+    '''
+    try:
+      chain = self.chain_refs[target]
+      del chain[perk]
+    except (KeyError, ValueError):
+      pass
+
+  def addTaskRef(self, key, perk):
     '''
     Adds a key that can be used later to quickly look up a reference to a
     L{Perk} in this L{Tier}. Optimization for the L{getKeyedTask},
@@ -117,7 +205,7 @@ class Tier(object):
     '''
     self.perk_refs[key] = perk
     
-  def removePerkRef(self, key):
+  def removeTaskRef(self, key):
     '''
     Removes a key used to quickly look up a reference to a L{Perk} in this 
     L{Tier}. Keys are cleaned up automatically when L{Perk}s are destroyed.
@@ -178,7 +266,7 @@ class Tier(object):
       # initialize the Perk base classes with important, persistent references
       perk.preInit(ae, self)
       # initialize this particular instance with temporary references
-      perk.preExecute(Task.LAYER_FOCUS)
+      perk.preExecute(AEConstants.LAYER_FOCUS)
       try:
         # now let Perk specific code run
         perk.init()
@@ -186,10 +274,14 @@ class Tier(object):
         # pop the perk from the stack if it fails to initialize
         self.perks.pop(index)
         # let it finalize
-        perk.close()
+        try:
+          perk.close()
+        except Exception:
+          # ignore errors while finalizing
+          pass
         perk.postClose()
-        # log the exception
-        log.exception('Error in Perk.init')
+        # warn about the error
+        log.debug('Cannot initialize %s: %s', perk.getClassName(), str(e))
       perk.postExecute(False)
     
   def popPerk(self, *indices):
@@ -207,16 +299,26 @@ class Tier(object):
     # have to pop in reverse order to avoid corrupting original indices
     indices.sort(reverse=True)
     perks = map(self.perks.pop, indices)
-    map(Perk.Perk.close, perks)
-    map(Perk.Perk.postClose, perks)
+    for perk in perks:
+      try:
+        perk.close()
+      except Exception:
+        # ignore errors during close
+        pass
+      perk.postClose()
 
   def clearPerks(self):
     '''
     Removes all L{Perk}s from the stack. Calls L{Perk.Perk.close} on each to 
     ensure they have a chance to persist state.
     '''
-    map(Perk.Perk.close, self.perks)
-    map(Perk.Perk.postClose, self.perks)
+    for perk in self.perks:
+      try:
+        perk.close()
+      except Exception:
+        # ignore errors during close
+        pass
+      perk.postClose()
     self.perks = []
       
   def getPerks(self):
@@ -289,15 +391,15 @@ class Tier(object):
     @param gesture_list: Gestures and device expressed as a list of virtual key 
       codes
     @type gesture_list: L{AEInput.Gesture}
-    @return: L{Task} set to execute in response to the input gesture or None
-    @rtype: L{Task.InputTask.InputTask}
+    @return: Name of the L{Task} to execute in response to the input gesture 
+      or None if no L{Task} registered to execute
+    @rtype: string
     '''
     try:
       perk = self.perk_refs[gesture_list]
     except KeyError:
       return None
-    name = perk.getCommandTask(gesture_list)
-    return self.getNamedTask(name)
+    return perk.getCommandTask(gesture_list)
 
   def getNamedTask(self, name):
     '''
@@ -325,18 +427,38 @@ class Tier(object):
 
     @param event_type: Desired type of L{AEEvent}
     @type event_type: L{AEEvent} class
-    @param task_layer: Layer on which the desired L{Task}s are registered, one 
-      of L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param task_layer: Layer on which the desired L{Task}s are registered
     @type task_layer: integer
     @return: List of all L{Task}s registered to handle the given type of event
       on the given layer
     @rtype: list of L{Task.Base.Task}
     '''
     tasks = []
-    map(tasks.extend, [p.getEventTasks(event_type, task_layer) for p in perks])
+    map(tasks.extend, [p.getEventTasks(event_type, task_layer) for p in 
+                       self.perks])
     return tasks
   
-  def _executeTask(self, task, por, layer, task_data, propagate, timestamp=0):
+  def getChainedTasks(self, target, link):
+    '''
+    Gets all L{Task}s linked to the target according to the given link type by
+    iterating through L{Perk} references established by L{addChainRef}. The
+    L{Task}s are returned in the order in which they were registered within a
+    L{Perk}. Ordering is undefined across L{Perk}s within a chain segment.
+
+    @param target: Name of the L{Task} to link to
+    @type target: string
+    @param link: One of the CHAIN constants in L{AEConstants.Tools} 
+    @type link: integer
+    '''
+    tasks = []
+    try:
+      perks = self.chain_refs[target].keys()
+    except KeyError:
+      return tasks
+    map(tasks.extend, [p.getChainSegment(target, link) for p in perks])
+    return tasks   
+  
+  def _executeTask(self, task, por, layer, params, propagate):
     '''
     Executes the given L{Task} in response to the given L{AEEvent}.
     
@@ -346,30 +468,28 @@ class Tier(object):
     @type por: L{POR}
     @param propagate: Should this event be propagated to the 
       L{Task.Base.Task.execute} method or should we call 
-      L{Task.Base.Task.update} instead?
+      L{Task.Base.Task.update} instead for housekeeping?
     @type propagate: boolean
     @param layer: Layer on which the L{AEEvent} occurred, one of 
-      L{AEEvent.LAYER_FOCUS}, L{AEEvent.LAYER_TIER}, 
-      L{AEEvent.LAYER_BACKGROUND}
+      L{AEConstants.LAYER_FOCUS}, L{AEConstants.LAYER_TIER}, 
+      L{AEConstants.LAYER_BACKGROUND}
     @type layer: integer
-    @param task_data: Keyword arguments to be provided to the L{Task}
-    @type task_data: dictionary
-    @param timestamp: Time at which the event occurred. Not available for all
-      events. Defaults to zero.
-    @type timestamp: float
-    @return: Should the next registered L{Task} be executed?
+    @param params: Keyword arguments to be provided to the L{Task} as 
+      parameters
+    @type params: dictionary
+    @return: Should the next registered L{Task} be executed or only updated?
     @rtype: boolean
     '''
     # pass important references to the Task
-    if not task.preExecute(layer, por, timestamp):
+    if not task.preExecute(layer, por):
       return True
     try:
       if propagate:
         # unpack the dictionary and execute the task
-        rv = task.execute(layer=layer, **task_data)
+        rv = task.execute(layer=layer, **params)
       else:
         # unpack the dictionary and update the task
-        task.update(layer=layer, **task_data)
+        task.update(layer=layer, **params)
         rv = False
     except Task.ToolsError, ex:
       # speak the Task.ToolsError if state.Trap is True
@@ -380,7 +500,7 @@ class Tier(object):
       rv = True
     except Exception:
       # log any other Perk exception
-      log.exception('%s: %s %s', task.getClassName(), layer, task_data)
+      log.exception('%s: %s %s', task.getClassName(), layer, params)
       # continue processing
       rv = True
     if rv is None:
@@ -388,7 +508,7 @@ class Tier(object):
       rv = True
     # uninitialize the Task, potentially storing the POR only if the event was
     # in the focus layer and execution is still propagating
-    task.postExecute(task.layer == Task.LAYER_FOCUS and propagate)
+    task.postExecute(task.layer == AEConstants.LAYER_FOCUS and propagate)
     return rv
   
   def manageEvent(self, event):
@@ -430,7 +550,7 @@ class Tier(object):
     # get these values once to initialize, most are constant for all Tasks
     event_por = event.getPOR()
     task_layer = event.getLayer()
-    task_data = event.getDataForTask()
+    task_params = event.getDataForTask()
     event_type = type(event)
 
     # run through all the registered Perks
@@ -442,11 +562,11 @@ class Tier(object):
       # run through all Tasks of the given type in this Perk
       for task in perk.getEventTasks(event_type, task_layer):
         propagate = self._executeTask(task, event_por, task_layer, 
-                                      task_data, propagate)
+                                      task_params, propagate)
         self.tier_manager.showTask(event, perk, task, propagate)
       perk.postExecute(False)
         
-  def manageGesture(self, event):
+  def manageGesture(self, event, count):
     '''
     Manages an event by getting the L{AEInput.Gesture} that triggered it and
     locating a L{Task} registered to execute in response to it. If a L{Task} 
@@ -455,25 +575,29 @@ class Tier(object):
 
     @param event: Event to process
     @type event: L{AEEvent.InputGesture}
+    @param count: Number of times this gesture has been issued without 
+      interruption
+    @type count: integer
     '''
     # look for a Perk with a Task registered to respond to this input command
-    task = self.getCommandTask(event.getTaskKey())
+    name = self.getCommandTask(event.getTaskKey())
+    if name is None: return
+    
     # show the event in registered monitors
     self.tier_manager.showEvent(event, self.name)
-    if task is not None:
-      layer = event.getLayer()
-      timestamp = event.getTimestamp()
-      perk = task.getPerk()
-      if not perk.preExecute(layer):
-        return
-      # execute that Task
-      self._executeTask(task, None, layer, {}, True, timestamp)
-      perk.postExecute(False)
+    # extract layer and data for task
+    layer = event.getLayer()
+    task_params = event.getDataForTask()
+    # insert the count into the data for the task
+    task_params['cycle_count'] = count
+    # use manageNamedTask to account for chains on tasks
+    self.manageNamedTask(None, name, layer, task_params, True)
       
   def manageKeyedTask(self, event):
-    '''
-    Manages an event by locating a L{Task} under a particular key registered to
-    execute in response to it.
+    '''    
+    Manages an event by locating a L{Task} under a particular key (not keyboard
+    key, but some immutable identifier) registered to execute in response to
+    it.
 
     @param event: Event to process
     @type event: L{AEEvent}
@@ -485,38 +609,76 @@ class Tier(object):
     if task is not None:
       layer = event.getLayer()
       perk = task.getPerk()
-      task_data = event.getDataForTask()
-      if not perk.preExecute(layer):
+      task_params = event.getDataForTask()
+      if not perk or not perk.preExecute(layer):
         return
       # execute that Task
-      self._executeTask(task, None, layer, task_data, True)
+      self._executeTask(task, None, layer, task_params, True)
       perk.postExecute(False)
 
-  def manageNamedTask(self, por, name, layer, task_data):
+  def manageNamedTask(self, por, name, layer, task_params, propagate=True,
+                      chain=True):
     '''
     Executes a L{Task} registered under the given name in this L{Tier} by
     configuring it to run in the given layer and with the given keyword 
-    argument data.
+    argument data. Follows any chained L{Task}s and recurisvely executes them 
+    before, around, and after. 
     
-    @todo: PP: show doTask?
+    @todo: PP: show doTask? self.tier_manager.showEvent(event, self.name)
+    @todo: PP: implement support for around
     
     @param name: Name of the L{Task} to execute
     @type name: string
     @param layer: Layer in which to execute the L{Task}, one of 
-      L{AEEvent.LAYER_FOCUS}, L{AEEvent.LAYER_TIER}, 
-      L{AEEvent.LAYER_BACKGROUND}
+      L{AEConstants.LAYER_FOCUS}, L{AEConstants.LAYER_TIER}, 
+      L{AEConstants.LAYER_BACKGROUND}
     @type layer: integer
-    @param task_data: Keyword data to pass to the L{Task} on execution
-    @type task_data: dictionary
+    @param task_params: Keyword data to pass to the L{Task} as parameters on 
+      execution or update
+    @type task_params: dictionary
+    @param propagate: Should chained L{Task}s be executed or updated?
+    @type propagate: boolean
+    @param chain: Execute all L{Task}s chained to the one named or not?
+    @type chain: boolean
     '''
     # look for a Perk with a Task registered to respond to this event
     task = self.getNamedTask(name)
-    # show the event in registered monitors
-    #self.tier_manager.showEvent(event, self.name)
-    if task is not None:
-      perk = task.getPerk()
-      if not perk.preExecute(layer, por):
-        return
-      # execute that Task
-      self._executeTask(task, por, layer, task_data, True)
+    # bail immediately if it doesn't exist
+    if task is None: return
+    
+    if chain:
+      # get the chain of tasks to run before this one
+      for before_name in self.getChainedTasks(name, AEConstants.CHAIN_BEFORE):
+        # recurse with before tasks
+        self.manageNamedTask(por, before_name, layer, task_params, propagate)
+        # use the pointer as the input to the next task
+        por = self.getPointer()
+      
+    perk = task.getPerk()
+    if perk.preExecute(layer, por):
+      # execute this Task
+      self._executeTask(task, por, layer, task_params, propagate)
       perk.postExecute(False)
+    
+    if chain:
+      # get the chain of tasks to run after this one
+      for after_name in self.getChainedTasks(name, AEConstants.CHAIN_AFTER):
+        # recurse with before tasks
+        self.manageNamedTask(por, after_name, layer, task_params, propagate)
+        # use the pointer as the input to the next task
+        por = self.getPointer()
+
+  def managePrivate(self, event):
+    '''
+    Pulls private data from an event and stores it in appropriate instance
+    variables. Does not forward the event to L{Perk}s and L{Task}s. Rather,
+    the stored information is made public through L{Task.Tools} methods.
+    
+    @param event: Event to process
+    @type event: L{AEEvent}
+    '''
+    data = event.getDataForTask()
+    kind = data['kind']
+    if kind == event.KEY_PRESS:
+      # store last scan code and key sym string
+      self.last_key = data['code'], data['sym'], data['mod']
