@@ -51,18 +51,18 @@ constructor.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import os, glob, imp, cPickle, sys, logging, shutil
 from SettingsManager import SettingsManager
 from i18n import _
-from LSRConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
+from AEConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
      BUILTIN_PROFILES
 
 # create a logger for UIRegistrar
@@ -638,16 +638,19 @@ def disassociate(name, profiles=None, tier=None, all_tiers=False):
       # create a new UIESet if it does not exist for this profile
       uieset = UIESet(profile)
     disassociated.append(profile)
-    # remove from all if the metadata is no longer available
-    if metadata is None:
-      uieset.removeFromAll(name)
-    # decide when not to load the given UIE
-    elif metadata.kind in STARTUP_KINDS:
-      uieset.removeFromStartup(metadata.kind, name)
-    elif all_tiers:
-      uieset.removeFromAnyTier(metadata.kind, name)
-    elif tier is not None:
-      uieset.removeFromTier(metadata.kind, name, tier)     
+    try:
+      # remove from all if the metadata is no longer available
+      if metadata is None:
+        uieset.removeFromAll(name)
+        # decide when not to load the given UIE
+      elif metadata.kind in STARTUP_KINDS:
+        uieset.removeFromStartup(metadata.kind, name)
+      elif all_tiers:
+        uieset.removeFromAnyTier(metadata.kind, name)
+      elif tier is not None:
+        uieset.removeFromTier(metadata.kind, name, tier)     
+    except ValueError:
+      pass
     # write the profile back to disk
     sm.save(__name__, uieset)
   return disassociated
